﻿/// <reference path="../ext/knockout-2.1.0.debug.js" />
(function (global, $, ko) {


    ko.externalTemplates = (function () {

        var settings = {
            path: "/KoTemplates",
            extension: "htm",
            loading: "<p>Loading...</p>"
        };

        var templates = {};

        var externalTemplates = function (options) {
            settings = $.extend(settings, options);
        };

        externalTemplates.get = function (name) {
            if (!templates[name])
                templates[name] = new ko.templateSources.external(name, settings);
            return templates[name];
        };

        return externalTemplates;

    }());

    // Define a new external template source
    var externalTemplateSource = ko.templateSources.external = function (name, options) {

        var self = this;
        var originalAfterRender = options.afterRender;

        self.template = ko.observable(options.loading);
        self.template.data = {};

        // Load the template
        $.ajax({
            url: options.path + "/" + name + "." + options.extension,
            dataType: "html",
            success: function (html, textStatus, jqXHR) {
                self.text(html);
                if (originalAfterRender)
                    originalAfterRender();
            },
            error: function (jqXHR, textStatus, errorThrown) {
                self.text("<p>Error (" + jqXHR.status + ") loading template : " + url + "</p>");
            },
            cache: false
        });

    };

    externalTemplateSource.prototype["text"] = function (value) {
        if (arguments.length === 0)
            return this.template();
        this.template(value);
    };

    externalTemplateSource.prototype["data"] = function (key, value) {
        if (arguments.length === 1) {
            if (key === "precompiled")
                this.template();
            return this.template.data[key];
        }
        this.template.data[key] = value;
    };

    // Define a new template engine that can load external templates
    var externalTemplateEngine = function () {

        var engine = new ko.nativeTemplateEngine();

        engine.makeTemplateSource = function (template, bindingContext, options) {

            var templateSource;

            // Named template
            if (typeof template == "string") {

                var elem = document.getElementById(template);

                if (elem)
                    return new ko.templateSources.domElement(elem);
                else
                    return ko.externalTemplates.get(template);


            }
            else if ((template.nodeType == 1) || (template.nodeType == 8)) {
                    // Anonymous template
                return new ko.templateSources.anonymousTemplate(template);
            }

        };

        engine.renderTemplate = function (template, bindingContext, options) {
            var templateSource = engine.makeTemplateSource(template, bindingContext, options);
            return engine.renderTemplateSource(templateSource, bindingContext, options);
        };

        return engine;

    }

    ko.externalTemplateEngine = new externalTemplateEngine();
    ko.setTemplateEngine(ko.externalTemplateEngine);

}(window, jQuery, ko));
