﻿(function (global, $, ko) {

    ko.viewModels = (function () {

        // ViewModels:
        //  {
        //      viewModel: {},
        //      callbacks: []
        //  }

        var settings = {
            path: "KoViewModels",
            extension: "js"
        };

        var viewModels = function (options) {
            $.extend(settings, options);
        };

        var viewModel = function (name) {
            // encapsulate the loading of view models
            // into this object.
            // need to handle callbacks from multiple sources.
            var self = this;

            self.name = name;
            self.loaded = false;
            self.loading = false;
            self.callbacks = [];
            self.viewModel = {};

            self.registerCallback = function (callback) {

                if (self.loaded)
                    return callback(self.viewModel);

                self.callbacks.push(callback);

                if (!self.loading) {

                    self.loading = true;

                    $.getScript(settings.path + "/" + self.name + "." + settings.extension, function (script) {

                        self.viewModel = eval(script);
                        if (typeof self.viewModel["init"] === "function")
                            self.viewModel["init"]();

                        // Finished loading
                        self.loaded = true;
                        self.loading = false;

                        // Return the view model to each of the requestors
                        ko.utils.arrayForEach(self.callbacks, function (x) {
                            x(self.viewModel);
                        });

                    });
                }

            };

        };

        viewModels.get = function (name, callback) {
            viewModels[name] = viewModels[name] || new viewModel(name);
            viewModels[name].registerCallback(callback);
        };

        return viewModels;

    }());


    // Wrap the template binding handler to enable loading of external view models
    var oldHandler = ko.bindingHandlers.template;
    ko.bindingHandlers["template"] = {
        init: function (element, valueAccessor) {
            return oldHandler.init(element, valueAccessor);
        },
        update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
            var val = valueAccessor();
            if (typeof val === "object" && "viewModel" in val) {
                // Requires the retrieval of an external view model
                ko.viewModels.get(val["viewModel"], function (vm) {
                    bindingContext.$data = vm;
                    viewModel = vm;
                    return oldHandler.update(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
                });
            }
            else {
                return oldHandler.update(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext);
            }
        }
    };

}(window, jQuery, ko));
